#!/usr/bin/perl

#
# merlin_qtl.pl :  Prepares Merlin output for displaying with gnuplot
# Syntax    :  merlin_qtl_plot.pl <options>
# Infiles   :  merlin_<qtl>.out
# Outfiles  :  myplot.gp, chr_line.plt, <qtl>.plt and merlin_<qtl>.ps
# 
# Created 12-May-2005 FR 
# Modified 26-May-2005 FR

use Cwd;
use strict;

#######################  G L O B A L  V A R I A B L E S  #######################

my $sel="qtl"; 	# selection of analysis	[qtl|dev|vc|cov]
my $score="";

my $maxlines = 999999; 	# Maximum number of lines (input-file)

my $xsum = 0.0;
my $xmin = 0;
my $ymax = 2;
my $ymin = -1;
my $lastxsum=0.;
my @xsums = ();
my %chrlabel=();
my $pfad = cwd();
my $infile="";
my $datum = scalar localtime;

my $deletes=1;
my $hline=1;
my $legend=1;

####################################  M A I N  #################################

for (my $i=0;$i<=$#ARGV;$i++) {
	if ($ARGV[$i] =~ "-h" ) { &Help;}
	elsif ($ARGV[$i] =~ "-clean" ) { &clean;}
	elsif ($ARGV[$i] =~ "-nd" ) { $deletes=0;}
	elsif ($ARGV[$i] =~ "-nohline" ) {$hline=0;}
	elsif ($ARGV[$i] =~ "-nl" ) {$legend=0;}
	elsif ($ARGV[$i] =~ "-qtl" ) { $sel="qtl";}
	elsif ($ARGV[$i] =~ "-dev" ) { $sel="dev";}
	elsif ($ARGV[$i] =~ "-vc" ) { $sel="vc";}
	elsif ($ARGV[$i] =~ "-cov" ) { $sel="cov";}
	else { &Help;}
}

print "\n\n";
print " * * * * * * * * * * *  MERLIN_QTL_PLOT.PL v1.0  * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n\n";
print "Please wait ...\n\n";

open (OUT1,"> z.plt") || die "could not open z.plt :$!\n";
open (OUT4,"> lod.plt") || die "could not open lod.plt :$!\n";

foreach my $dir qw(c01 c02 c03 c04 c05 c06 c07 c08 c09 c10 c11 c12 c13 c14 c15 c16 c17 c18 c19 c20 c21 c22 c23 c24) {

	$infile = "$dir/merlin_qtl.out";

	if (-f "$infile") {
		print "File: $infile "; 
		&readqtl;

		my $xpos=(($lastxsum + $xsum)/2);
		my $var=$dir; $var =~ s/c0//;$var =~ s/c//;
		if ($var eq "23"){$var = "X";}
		if ($var eq "24"){$var = "XY";}
		if ($var > 9){ $xpos -=30;}
		else { $xpos -=5;}
		$chrlabel{$var}=$xpos;	  
		$lastxsum=$xsum;

		push (@xsums,$xsum);
	}
	else { print "$infile not exists \n"; }
} # end foreach
close (OUT1);
close (OUT4);

&print_chr_line;
&pr_batch;

# print "\nYou can now view results by \"gnuplot myplot.gp\" \n\n";
system "gnuplot  myplot.gp";

if ($deletes == 1) {&clean;}


#######################  R E A D   Q T L - S C O R E S  #######################

sub readqtl {
	open (infile, "$infile") || die "could not open $infile :$!\n";

	my $jetzt = $maxlines;
	my $xmax = 0.0;
	my $hit=0;
	my $now=0;
	my $count=0;

	while (<infile>){
		my @feld =	split(' ',$_);
		$count++;
		if ($feld[0] =~ /Phenotype:/)   {
			if ((($feld[2] eq "\[QTL\]") && ($sel eq "qtl"))
			|| (($feld[2] eq "\[QTL\,") && ($sel eq "dev"))
			|| (($feld[2] eq "\[VC\]") && ($sel eq "vc"))
			|| (($feld[2] eq "\[cov\]") && ($sel eq "cov"))) {
				$score=$feld[2];
				$hit=1; 
				$now=1;
				print "  score: $score";
			}
		}
		if ($feld[0] =~ /Skipping/) {print "\t$_\n";}

		if (($feld[0] =~ /max/) && ($now == 1)) { $jetzt = $count;}	# begin of lod data
		if (($feld[2] =~ /ChiSq/) && ($now == 1)) { $jetzt = $count;}	# begin of lod data
		if ($#feld < 0) { $jetzt = $maxlines; $now=0;}		# end of lod data 

		if ($count > $jetzt) {
			if (($sel eq "qtl") || ($sel eq "dev")) {
				printf(OUT1 "%f %s\n", ($feld[0] + $xsum), $feld[1]);
				printf(OUT4 "%f %s\n", ($feld[0] + $xsum), $feld[4]);
				if ($feld[0] > $xmax) { $xmax = $feld[0];}  
				if ($feld[1] > $ymax) { $ymax = $feld[1];}
			}
			elsif (($sel eq "vc") || ($sel eq "cov")) {
				printf(OUT1 "%f %s\n", ($feld[0] + $xsum), $feld[2]);
				printf(OUT4 "%f %s\n", ($feld[0] + $xsum), $feld[3]);
				if ($feld[0] > $xmax) { $xmax = $feld[0];}  
				if ($feld[2] > $ymax) { $ymax = $feld[2];}
			}		
			else {;}
		}

	} # end while
	if ($hit == 0) {print "  no analysis\n";}
	else {print "\n";}
	$xsum = $xsum + $xmax;
	printf OUT1 "\n";
	printf OUT4 "\n";

  close (infile);

} ## end readqtl


####################  P R I N T  G N U P L O T  B A T C H   ####################
sub pr_batch {
	my $co_1 = 1;
	my $co_2 = 3;
    my $titel="";
    my $t1="Zmean";
    my $t2="LOD";
    
	if ($sel eq "qtl") {$titel = "$pfad - Merlin  QTL - $datum";}
	elsif ($sel eq "dev") {$titel = "$pfad - Merlin  deviates - $datum";}
	elsif ($sel eq "vc") {$titel = "$pfad - Merlin  variance component - $datum";$t1="Chi\^2";}
	elsif ($sel eq "cov") {$titel = "$pfad - Merlin  useCovariates - $datum";$t1="Chi\^2";}
	else {;}
	
	open (OUT2,"> myplot.gp") || die "could not open myplot.gp :$!\n";

	print OUT2 "set autoscale\n";
	print OUT2 "set title \"$titel\\n\"\n" ;
	print OUT2 "set xlabel \"cM\"\n";
	print OUT2 "set ylabel \"${t1}/$t2\"\n";
	my $ypos = $ymax + (($ymax-$ymin)/23);
	foreach (keys %chrlabel) {print OUT2 "set label \"$_\" at $chrlabel{$_}\,$ypos\n";}
	print OUT2 "set size 1,0.8\n";
	if ($legend == 0){ print OUT2 "set nokey\n";}
	print OUT2 "set terminal postscript color solid 11\n";
	print OUT2 "set output \"merlin_${sel}.ps\"\n";
	print OUT2 "plot \[$xmin\:$xsum\] \[$ymin\:$ymax\]   \'chr_line.plt\' notitle with lines lt 0, \'z.plt\' title \"$t1\" with lines 3, \'lod.plt\' title \"$t2\"with lines lt 1\n";
	print OUT2 "set terminal windows\n";
	print OUT2 "replot\n";
	print OUT2 "\npause -1 \"Hit return to continue\"\n\n";

	close (OUT2);
}

######################   P R I N T  C H R _ L I N E S   ######################
sub print_chr_line {
	$ymax += 1;			# y-achse auf ganze Zahl bringen
	my @buff =	split('\.',$ymax);
	$ymax=$buff[0];

	open (OUT3,"> chr_line.plt") || die "could not open chr_line.plt :$!\n";

	if ($hline=1){
		for (my $i=1; $i<$ymax;$i++) {	# Linien bei LODs = 1,2,3,4,..
			print OUT3 "0 $i\n$xsum $i\n\n";
		}
	}
	pop @xsums;
	foreach (@xsums){	# Linien Chromosomenende 1,2,3,4,...22,x,xy
		print OUT3 "$_ $ymin \n$_ $ymax \n\n";         
	}
	close (OUT3);
}


##################################  H E L P  ############################
sub Help {
print "\n\n";
print " * * * * * * * * * * *  MERLIN_QTL_PLOT.PL v1.1  * * * * * * * * * * *\n";
print " * * * * * * * * * View Merlin results with GNUPLOT  * * * * * * * * *\n";
print " * * * * * * * *  Created by Franz Rueschendorf  2005  * * * * * * * *\n";
print " * * * * * * * * * *  Max-Delbrueck-Center, Berlin * * * * * * * * * *\n";
print " * * * * * * * * * *  email: fruesch\@mdc-berlin.de * * * * * * * * * *\n\n";
print "usage        : perl merlin_qtl_plot.pl [-<options>]\n";
print "infiles      : merlin_<qtl>.out\n";
print "outfiles     : myplot.gp, <qtl>.plt, lod.plt, chr_line.plt, merlin_<qtl>.ps\n" ;
print "view results : gnuplot myplot.gp\n\n";
print "options      :\n";
print "\t-h\thelp \(this\)\n";
print "\t-clean\tdelete temporary files\n";
print "\t-qtl\tplot analysis --qtl \(default\)\n";
print "\t-dev\tplot analysis --deviates\n";
print "\t-vc\tplot analysis --vc\n";
print "\t-cov\tplot analysis --useCovariates\n";
exit 0;
}

###################################  C L E A N  ################################

sub clean { 
unlink glob("*.plt");
unlink glob("*.gp");
exit 0;
}

### end merlin_qtl.pl
